function [LB] = BananaLow(model,IntX)

% -------------------------------------------------------------------------
% SPIKE_O toolbox - BananaLow.m
%
% This function evaluates a lower bound to banana function
% 
% Syntax:
%   [LB] = BananaLow(model,IntX) 
%
% Inputs:
%   model   :   Structure describing the parameters of the banana function
%   IntX    :   Polyhedral set of feasible parameters
%
% Outputs:
%   LB      :   lower bound banana function value
%
% -------------------------------------------------------------------------
% Last modification: Kris Villez, 2013-05-31
% -------------------------------------------------------------------------

a =  model.a; 
b =  model.b; 
c = model.c;

IntX1 = [ max(a-IntX(:,1)) ; min(a-IntX(:,1)) ] ;
if all(IntX1>=0) || all(IntX1<=0) 
    IntX1 = [ max(IntX1.^2) ; min(IntX1.^2) ] ;
else
    IntX1 = [ max(IntX1.^2) ; 0 ] ;
end

IntX2 = [ max(b-IntX(:,1)) ; min(b-IntX(:,1)) ] ;
if all(IntX2>=0) || all(IntX2<=0) 
    IntX2 = [ max(IntX2.^2) ; min(IntX2.^2) ] ;
else
    IntX2 = [ max(IntX2.^2) ; 0 ] ;
end

IntXY = [   max([IntX(1,2)-IntX2(:,1) ; IntX(2,2)-IntX2(:,1)]) ;...
            min([IntX(1,2)-IntX2(:,1) ; IntX(2,2)-IntX2(:,1)]) ] ;
if all(IntXY>=0) || all(IntXY<=0) 
    IntXY = [ max(IntXY.^2) ; min(IntXY.^2) ] ;
else
    IntXY = [ max(IntXY.^2) ; 0 ] ;
end

IntZ = [   max([IntX1(1,1)+c*IntXY(:,1) ; IntX1(2,1)+c*IntXY(:,1)]) ;...
            min([IntX1(1,1)+c*IntXY(:,1) ; IntX1(2,1)+c*IntXY(:,1)]) ] ;
LB = IntZ(2) ;