function fathom      =   Fathoming(Tree,Alive)  

% -------------------------------------------------------------------------
% SPIKE_O toolbox - Fathoming.m
%
% This function evaluates which nodes are to be fathomed as part of the
% branch-and-bound algorithm
%
% Syntax:
%   fathom  =   Fathoming(Tree,Alive)
%
% Inputs:
%   Tree    :   Solution tree
%   Alive   :   Boolean vector indicating whether nodes are alive or not
%
% Outputs:
%   fathom  :   List of nodes to be fathomed
%
% -------------------------------------------------------------------------
% Latest version: Kris Villez, 2013-05-14
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% Copyright 2012-2013 Kris Villez
%
% This file is part of the SPIKE_O Toolbox for Matlab/Octave. 
% 
% The SPIKE_O Toolbox is free software: you can redistribute it and/or
% modify it under the terms of the GNU General Public License as published
% by the Free Software Foundation, either version 3 of the License, or (at
% your option) any later version.
% 
% The SPIKE_O Toolbox is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
% Public License for more details.
% 
% You should have received a copy of the GNU General Public License along
% with the SPIKE_O Toolbox. If not, see <http://www.gnu.org/licenses/>.  
% -------------------------------------------------------------------------


% column indices for lower bound and upper bound:
iLB =   4   ;
iUB =   5   ;

nAlive          =   length(Alive)       ;   % number of live nodes
fathom          =   zeros(nAlive,1)     ;   % set fathom to 0/false (unmarked)
for j = 1:nAlive    % for each live node
    lower       =   Tree(Alive(j),iLB)  ;   % lower bound corresponding to current node
    % if there is any other node which has an upper bound lower than this
    % lower bound, then mark this live node to be fathomed (1/true)
    OthersAlive  =   setdiff(Alive,Alive(j)) ;
    if ~isempty(OthersAlive)
        fathom(j)   =   min(Tree(OthersAlive,iUB))+1e-6<lower   ;
    end
end
if all(fathom)
    Tree(Alive,iLB)
    Tree(Alive,iUB)
end
fathom          =   find(fathom)        ;   % list nodes to be fathomed
fathom          =   unique(fathom)      ;   % list as unique nodes

% nAlive          =   length(Alive)       ;   % number of live nodes
